#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <error.h>
#include <sys/stat.h>

unsigned int h_len;

unsigned int getword(unsigned char* data)
{
	return *data + 256 * (*(data + 1));
}

void tooshort(FILE *fd)
{
	printf("Input file too short.\n\n");
	fclose(fd);
	exit(2);
}

void decode(unsigned char *header)
{
	char name[11];
	unsigned int n;

	strncpy(name, (char*)(header+2), 10);
	name[10] = '\0';
	printf("Filename: %s\n", name);
	printf("Flag: %u\n", (unsigned int)*header);
	n = getword(header+14);
	h_len = getword(header+12);
	switch(*(header+1))
	{
		case '\0':
			printf("Type: 0 => program\nProgram length: %u bytes\n",
				h_len);
			if (n < 32768)
				printf("Runs from line %u\n", n);
			printf("Length without variables: %u bytes\n",
				getword(header+16));
			break;
		case '\1':
			printf("Type: 1 => number array\nLength: %u bytes\n",
				h_len);
			break;
		case '\2':
			printf("Type: 2 => character array\nLength: %u bytes\n",
				h_len);
			break;
		case '\3':
			if (n == 16384 && h_len == 6912)
				printf("Type: 3 => screen image\n");
			else
				printf("Type: 3 => bytes\n"
					"Start address: %u bytes\n"
					"Length: %u\n3rd param: %u\n",
					n, h_len, getword(header+16));
			break;
	}
	printf("\n");
}

int main(int argc, char** argv, char** env)
{
	FILE *fd;
	unsigned int err, no, l;
	long pos = 0;
	struct stat st;
	unsigned char header[19];

	if (argc != 2)
	{
		printf("Print information about tap file.\nUsage: %s file\n\n", argv[0]);
		return 1;
	}

	no = stat(argv[1], &st);
	if (no != 0)
	{
		err = errno;
		error(1, err, "can't stat input file");
	}

	fd = fopen(argv[1], "r");
	if (fd == NULL)
	{
		err = errno;
		error(1, err, "can't open input file");
	}

	while (pos < st.st_size)
	{
		pos += no = fread(header, 1, 2, fd);
		if (no != 2)
			tooshort(fd);

		no = getword(header);
		if (no == 19) /* HEADER */
		{
			pos += no = fread(header, 1, 19, fd);
			if (no != 19)
				tooshort(fd);
			decode(header);
		}
		else
		{
			if (h_len != no - 2) /* zobrazuj iba bloky bez hl. */
			{
				l = no;
				printf("Type: datablock\nLength: %u\n", l - 2);
				pos += no = fread(header, 1, 1, fd);
				if (no != 1)
					tooshort(fd);
				printf("Flag: %u\n\n", (int)*header);
				l--;
			}
			else
			{
				l = no;
			}

			pos += l;
			no = fseek(fd, pos, SEEK_SET);
			if (no != 0)
			{
				err = errno;
				error(1, err, "can't seek in input file");
			}
		}
	}

	fclose(fd);
	return 0;
}
